"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.bundleGraphToInternalBundleGraph = bundleGraphToInternalBundleGraph;
exports.default = void 0;

function _assert() {
  const data = _interopRequireDefault(require("assert"));

  _assert = function () {
    return data;
  };

  return data;
}

function _nullthrows() {
  const data = _interopRequireDefault(require("nullthrows"));

  _nullthrows = function () {
    return data;
  };

  return data;
}

var _Asset = require("./Asset");

var _Bundle = require("./Bundle");

var _Dependency = _interopRequireWildcard(require("./Dependency"));

var _Graph = require("../Graph");

var _utils = require("../utils");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Friendly access for other modules within this package that need access
// to the internal bundle.
const _bundleGraphToInternalBundleGraph = new WeakMap();

function bundleGraphToInternalBundleGraph(bundleGraph) {
  return (0, _nullthrows().default)(_bundleGraphToInternalBundleGraph.get(bundleGraph));
}

class BundleGraph {
  #graph;
  #options; // This is invoked as `this.#createBundle.call(null, ...)` below, as private
  // properties aren't currently callable in Flow:
  // https://github.com/parcel-bundler/parcel/pull/4591#discussion_r422661115
  // https://github.com/facebook/flow/issues/7877

  #createBundle;

  constructor(graph, createBundle, options) {
    this.#graph = graph;
    this.#options = options;
    this.#createBundle = createBundle; // $FlowFixMe

    _bundleGraphToInternalBundleGraph.set(this, graph);
  }

  getAssetById(id) {
    return (0, _Asset.assetFromValue)(this.#graph.getAssetById(id), this.#options);
  }

  getAssetPublicId(asset) {
    return this.#graph.getAssetPublicId((0, _Asset.assetToAssetValue)(asset));
  }

  isDependencySkipped(dep) {
    return this.#graph.isDependencySkipped((0, _Dependency.dependencyToInternalDependency)(dep));
  }

  getDependencyResolution(dep, bundle) {
    let resolution = this.#graph.getDependencyResolution((0, _Dependency.dependencyToInternalDependency)(dep), bundle && (0, _Bundle.bundleToInternalBundle)(bundle));

    if (resolution) {
      return (0, _Asset.assetFromValue)(resolution, this.#options);
    }
  }

  getIncomingDependencies(asset) {
    return this.#graph.getIncomingDependencies((0, _Asset.assetToAssetValue)(asset)).map(dep => new _Dependency.default(dep, this.#options));
  }

  getAssetWithDependency(dep) {
    let asset = this.#graph.getAssetWithDependency((0, _Dependency.dependencyToInternalDependency)(dep));

    if (asset) {
      return (0, _Asset.assetFromValue)(asset, this.#options);
    }
  }

  getBundleGroupsContainingBundle(bundle) {
    return this.#graph.getBundleGroupsContainingBundle((0, _Bundle.bundleToInternalBundle)(bundle));
  }

  getReferencedBundles(bundle, opts) {
    return this.#graph.getReferencedBundles((0, _Bundle.bundleToInternalBundle)(bundle), opts).map(bundle => this.#createBundle.call(null, bundle, this.#graph, this.#options));
  }

  getRequiredBundlesForBundle(bundle) {
    return this.#graph.getReferencedBundles((0, _Bundle.bundleToInternalBundle)(bundle)).map(bundle => this.#createBundle.call(null, bundle, this.#graph, this.#options));
  }

  resolveAsyncDependency(dependency, bundle) {
    let resolved = this.#graph.resolveAsyncDependency((0, _Dependency.dependencyToInternalDependency)(dependency), bundle && (0, _Bundle.bundleToInternalBundle)(bundle));

    if (resolved == null) {
      return;
    } else if (resolved.type === 'bundle_group') {
      return resolved;
    }

    return {
      type: 'asset',
      value: (0, _Asset.assetFromValue)(resolved.value, this.#options)
    };
  }

  getReferencedBundle(dependency, bundle) {
    let result = this.#graph.getReferencedBundle((0, _Dependency.dependencyToInternalDependency)(dependency), (0, _Bundle.bundleToInternalBundle)(bundle));

    if (result != null) {
      return this.#createBundle.call(null, result, this.#graph, this.#options);
    }
  }

  getDependencies(asset) {
    return this.#graph.getDependencies((0, _Asset.assetToAssetValue)(asset)).map(dep => new _Dependency.default(dep, this.#options));
  }

  isAssetReachableFromBundle(asset, bundle) {
    return this.#graph.isAssetReachableFromBundle((0, _Asset.assetToAssetValue)(asset), (0, _Bundle.bundleToInternalBundle)(bundle));
  }

  findReachableBundleWithAsset(bundle, asset) {
    let result = this.#graph.findReachableBundleWithAsset((0, _Bundle.bundleToInternalBundle)(bundle), (0, _Asset.assetToAssetValue)(asset));

    if (result != null) {
      return this.#createBundle.call(null, result, this.#graph, this.#options);
    }

    return null;
  }

  isAssetReferencedByDependant(bundle, asset) {
    return this.#graph.isAssetReferencedByDependant((0, _Bundle.bundleToInternalBundle)(bundle), (0, _Asset.assetToAssetValue)(asset));
  }

  hasParentBundleOfType(bundle, type) {
    return this.#graph.hasParentBundleOfType((0, _Bundle.bundleToInternalBundle)(bundle), type);
  }

  getBundlesInBundleGroup(bundleGroup) {
    return this.#graph.getBundlesInBundleGroup(bundleGroup).map(bundle => this.#createBundle.call(null, bundle, this.#graph, this.#options));
  }

  getBundles() {
    return this.#graph.getBundles().map(bundle => this.#createBundle.call(null, bundle, this.#graph, this.#options));
  }

  isEntryBundleGroup(bundleGroup) {
    return this.#graph.isEntryBundleGroup(bundleGroup);
  }

  getChildBundles(bundle) {
    return this.#graph.getChildBundles((0, _Bundle.bundleToInternalBundle)(bundle)).map(bundle => this.#createBundle.call(null, bundle, this.#graph, this.#options));
  }

  getParentBundles(bundle) {
    return this.#graph.getParentBundles((0, _Bundle.bundleToInternalBundle)(bundle)).map(bundle => this.#createBundle.call(null, bundle, this.#graph, this.#options));
  }

  resolveSymbol(asset, symbol, boundary) {
    let res = this.#graph.resolveSymbol((0, _Asset.assetToAssetValue)(asset), symbol, boundary ? (0, _Bundle.bundleToInternalBundle)(boundary) : null);
    return {
      asset: (0, _Asset.assetFromValue)(res.asset, this.#options),
      exportSymbol: res.exportSymbol,
      symbol: res.symbol,
      loc: (0, _utils.fromInternalSourceLocation)(this.#options.projectRoot, res.loc)
    };
  }

  getExportedSymbols(asset, boundary) {
    let res = this.#graph.getExportedSymbols((0, _Asset.assetToAssetValue)(asset), boundary ? (0, _Bundle.bundleToInternalBundle)(boundary) : null);
    return res.map(e => ({
      asset: (0, _Asset.assetFromValue)(e.asset, this.#options),
      exportSymbol: e.exportSymbol,
      symbol: e.symbol,
      loc: (0, _utils.fromInternalSourceLocation)(this.#options.projectRoot, e.loc),
      exportAs: e.exportAs
    }));
  }

  traverse(visit) {
    return this.#graph.traverse((0, _Graph.mapVisitor)(node => node.type === 'asset' ? {
      type: 'asset',
      value: (0, _Asset.assetFromValue)(node.value, this.#options)
    } : {
      type: 'dependency',
      value: new _Dependency.default(node.value, this.#options)
    }, visit));
  }

  traverseBundles(visit, startBundle) {
    return this.#graph.traverseBundles((0, _Graph.mapVisitor)(bundle => this.#createBundle.call(null, bundle, this.#graph, this.#options), visit), startBundle == null ? undefined : (0, _Bundle.bundleToInternalBundle)(startBundle));
  }

  findBundlesWithAsset(asset) {
    return this.#graph.findBundlesWithAsset((0, _Asset.assetToAssetValue)(asset)).map(bundle => this.#createBundle.call(null, bundle, this.#graph, this.#options));
  }

  findBundlesWithDependency(dependency) {
    return this.#graph.findBundlesWithDependency((0, _Dependency.dependencyToInternalDependency)(dependency)).map(bundle => this.#createBundle.call(null, bundle, this.#graph, this.#options));
  }

  getUsedSymbols(v) {
    if (v instanceof _Asset.Asset) {
      return this.#graph.getUsedSymbolsAsset((0, _Asset.assetToAssetValue)(v));
    } else {
      (0, _assert().default)(v instanceof _Dependency.default);
      return this.#graph.getUsedSymbolsDependency((0, _Dependency.dependencyToInternalDependency)(v));
    }
  }

}

exports.default = BundleGraph;